﻿package  
{
	import flash.display.BitmapData;
	import flash.display.DisplayObject;
	import flash.display.Loader;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.geom.Matrix;
	import flash.net.URLRequest;
	/**
	 * ...
	 * @author yves trublin
	 */
	public class PhotoLoader extends Sprite
	{
		private var bitmapDatas:Array;
		private var loading:Loading;
		private var _width:Number;
		private var _height:Number;
		private var iLoad:int;
		private var nbLoad:int;
		private var loader:Loader;
		private var trait:TraitLoading;
		
		public function PhotoLoader(w:Number, h:Number) 
		{
			_width = w;
			_height = h;

			if (stage) init();
			else addEventListener(Event.ADDED_TO_STAGE, init, false, 0, true);
		}
		
		private function init(e:Event = null):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, init);
			
			// tableau contenant les BitmapDatas
			bitmapDatas = [];
			iLoad = 0;
			
			// nombre de photos a charger
			nbLoad = root["xml"].Photo.length();
			
			initLoading();
		}

		private function initLoading():void
		{
			loader = new Loader();
			loader.contentLoaderInfo.addEventListener(IOErrorEvent.IO_ERROR, handlePhotoIOError, false, 0, true);
			loader.contentLoaderInfo.addEventListener(Event.COMPLETE, handlePhotoLoaded, false, 0, true);
			
			// loading
			loading = new Loading();
			loading.y = root["xml"].@["hauteurTraitChargement"] - 2;
			addChild(loading);

			// ajout trait loading
			trait = new TraitLoading();
			addChild(trait);
			
			// chargement premiere photo
			chargerPhoto();
			
			loader.contentLoaderInfo.addEventListener(ProgressEvent.PROGRESS, handlePhotoProgress, false, 0, true);			
		}
		
		private function handlePhotoProgress(e:ProgressEvent):void 
		{
			trait.miseAJourTraitLoading(loader.contentLoaderInfo.bytesLoaded / loader.contentLoaderInfo.bytesTotal);
		}
		
		private function chargerPhoto():void {
			loader.load(new URLRequest(root["xml"].Photo[iLoad].@["src"]));
			
			loading.miseAJourTexte(iLoad + 1);
		}
		
		private function handlePhotoLoaded(e:Event):void 
		{
			// on stock les bitmapData recadrees des photos dans un tableau
			var ratio:Number = Utils.calculerRatio(loader.content.width, loader.content.height, _width, _height) * (1 - root["xml"].@["reductionPhoto"]);

			var bmd:BitmapData = new BitmapData(loader.content.width * ratio, loader.content.height * ratio, false);
			bmd.draw(loader.content, new Matrix(ratio, 0, 0, ratio), null, null, null, true);
			bmd.lock();
			bitmapDatas[iLoad] = bmd;
			
			// on incremente l'indice de disponibilite des photo
			iLoad ++;
			
			if (iLoad < nbLoad) {
				chargerPhoto();
			}else {
				loader.contentLoaderInfo.removeEventListener(IOErrorEvent.IO_ERROR, handlePhotoIOError);
				loader.contentLoaderInfo.removeEventListener(Event.COMPLETE, handlePhotoLoaded);
				loader.contentLoaderInfo.removeEventListener(ProgressEvent.PROGRESS, handlePhotoProgress);
				loader = null;
				
				// on cache le loading
				loading.cacherLoading();
				
				trait.cacherTraitLoading();
			}

			// on indique au stage qu'une photo a ete chargee
			root.dispatchEvent(new Event("photo_chargee"));
		}
		
		private function handlePhotoIOError(e:IOErrorEvent):void 
		{
			loader.contentLoaderInfo.removeEventListener(IOErrorEvent.IO_ERROR, handlePhotoIOError);
			loader.contentLoaderInfo.removeEventListener(Event.COMPLETE, handlePhotoLoaded);
			loader.contentLoaderInfo.removeEventListener(ProgressEvent.PROGRESS, handlePhotoProgress);
		}

		//////////////////////////////////////////////////////////
		// getter / setter methods
		//////////////////////////////////////////////////////////
		public function get nombrePhotosDispo():int
		{
			return iLoad;
		}
		
		public function getBitmapData(i:int):BitmapData
		{
			return bitmapDatas[i];
		}
	}
}