package com.flashdynamix.motion {
	import flash.display.DisplayObject;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.ColorTransform;
	import flash.geom.Matrix;
	import flash.utils.getTimer;

	public class TweensyZeroMinified {
		public static var defaultTween : Function = easeOut;
		public static var onComplete : Function;
		private static var frame : Sprite = new Sprite();
		private static var time : int = 0;
		private static var list : Array = [];

		public static function to(instance : Object, to : Object, duration : Number = 0.5, ease : Function = null, update : Object = null, onComplete : Function = null) : TweensyTimelineZeroMinified {
			if(!hasTimelines) startUpdate();
			
			var timeline : TweensyTimelineZeroMinified = setup(instance, duration, ease, onComplete, update);
			
			for(var propName:String in to) {
				timeline.to[propName] = translate(instance[propName], to[propName]);
				timeline.properties++;
			}
			
			add(timeline);
			
			return timeline;
		}

		private static function add(item : TweensyTimelineZeroMinified) : void {
			
			var i : int = 0;
			var len : int = list.length;
			var timelineB : TweensyTimelineZeroMinified;
			var totalDuration : Number;
			
			for(i = len - 1;i >= 0; i--) {
				timelineB = list[i];
			
				totalDuration = timelineB.duration;
			
				if(timelineB.key == item.key && timelineB.time < totalDuration) {
					for(var propName:String in item.to) {
						delete timelineB.to[propName];
						timelineB.properties--;
					}
				
					if(timelineB.properties == 0) list.splice(i, 1);
				}
			}

			list.push(item);
		}

		private static function setup(instance : Object, duration : Number, ease : Function,  onComplete : Function, update : Object) : TweensyTimelineZeroMinified {
			
			var timeline : TweensyTimelineZeroMinified = new TweensyTimelineZeroMinified();
			timeline.instance = instance;
			timeline.duration = duration;
			timeline.ease = (ease != null) ? ease : defaultTween;
			timeline.update = update;
			timeline.onComplete = onComplete;
			timeline.key = (update != null) ? update : instance;
			
			return timeline;
		}

		/**
		 * Total number of animations in progress for the TweensyZeroMinified class.
		 */
		public static function get timelines() : int {
			return list.length;
		}

		/**
		 * Whether the TweensyZeroMinified Class has any timeline animations.
		 */
		public static function get hasTimelines() : Boolean {
			return (timelines > 0);
		}

		private static function update(event : Event) : void {
			var dif : Number = getTimer() - time;
			time += dif;
			dif *= 0.001;
			
			var i : int = 0;
			var len : int = list.length;
			var timeline : TweensyTimelineZeroMinified;
			var args : Array;
			
			for(i = len - 1;i >= 0; i--) {
				timeline = list[i];
				
				timeline.time += dif;
				
				var currentTime : Number = timeline.time;
				currentTime = (currentTime > timeline.duration) ? timeline.duration : currentTime;
				var finished : Boolean = currentTime >= timeline.duration;
				
				args = [currentTime, 0, 1, timeline.duration];
				
				var position : Number = timeline.ease.apply(null, args);
				var q : Number = 1 - position;
				
				
				for(var propName:String in timeline.to) {						
					timeline.instance[propName] = timeline.instance[propName] * q + timeline.to[propName] * position;
				}
				
				if(timeline.update != null) {
					if(timeline.instance is ColorTransform) {
						DisplayObject(timeline.update).transform.colorTransform = timeline.instance;
					} 
					else if (timeline.instance is Matrix) {
						DisplayObject(timeline.update).transform.matrix = timeline.instance;
					} 
				}
				
				if(finished) {
					if(timeline.onComplete != null) timeline.onComplete();
					
					list.splice(i, 1);
				}
			}
			
			if(!hasTimelines) {
				stopUpdate();
				if(onComplete != null) onComplete();
			}
		}

		private static function easeOut(t : Number, b : Number, c : Number, d : Number) : Number {
			return c * ((t = t / d - 1) * t * t * t * t + 1) + b;
		}

		private static function startUpdate() : void {
			time = getTimer();
			
			frame.addEventListener(Event.ENTER_FRAME, update, false, 0, true);
		}

		private static function stopUpdate() : void {
			frame.removeEventListener(Event.ENTER_FRAME, update);
		}

		private static function translate(current : Number, value : *) : Number {
			
			var target : Number;
			
			if(value is String) {
				var values : Array = String(value).split(",");
				if(values.length == 1) {
					target = current + parseFloat(value);
				} else {
					var start : Number = parseFloat(values[0]), end : Number = parseFloat(values[1]);
					target = current + start + (Math.random() * (end - start));
				}
			} else {
				target = value;
			}
			
			return target;
		}

	}
}
